import { HttpStatusCode } from '@peertube/peertube-models';
import { isUrlValid } from '../../../helpers/custom-validators/activitypub/misc.js';
import { CONFIG } from '../../../initializers/config.js';
import { body, param, query } from 'express-validator';
import { isBooleanValid, isIdValid, toBooleanOrNull } from '../../../helpers/custom-validators/misc.js';
import { isVideoChannelDescriptionValid, isVideoChannelDisplayNameValid, isVideoChannelSupportValid, isVideoChannelUsernameValid } from '../../../helpers/custom-validators/video-channels.js';
import { ActorModel } from '../../../models/actor/actor.js';
import { VideoChannelModel } from '../../../models/video/video-channel.js';
import { areValidationErrors, checkUserQuota, doesChannelHandleExist } from '../shared/index.js';
import { doesVideoChannelSyncIdExist } from '../shared/video-channel-syncs.js';
export const videoChannelsAddValidator = [
    body('name')
        .custom(isVideoChannelUsernameValid),
    body('displayName')
        .custom(isVideoChannelDisplayNameValid),
    body('description')
        .optional()
        .custom(isVideoChannelDescriptionValid),
    body('support')
        .optional()
        .custom(isVideoChannelSupportValid),
    async (req, res, next) => {
        if (areValidationErrors(req, res))
            return;
        const actor = await ActorModel.loadLocalByName(req.body.name);
        if (actor) {
            res.fail({
                status: HttpStatusCode.CONFLICT_409,
                message: 'Another actor (account/channel) with this name on this instance already exists or has already existed.'
            });
            return false;
        }
        const count = await VideoChannelModel.countByAccount(res.locals.oauth.token.User.Account.id);
        if (count >= CONFIG.VIDEO_CHANNELS.MAX_PER_USER) {
            res.fail({ message: `You cannot create more than ${CONFIG.VIDEO_CHANNELS.MAX_PER_USER} channels` });
            return false;
        }
        return next();
    }
];
export const videoChannelsUpdateValidator = [
    body('displayName')
        .optional()
        .custom(isVideoChannelDisplayNameValid),
    body('description')
        .optional()
        .custom(isVideoChannelDescriptionValid),
    body('support')
        .optional()
        .custom(isVideoChannelSupportValid),
    body('bulkVideosSupportUpdate')
        .optional()
        .custom(isBooleanValid).withMessage('Should have a valid bulkVideosSupportUpdate boolean field'),
    (req, res, next) => {
        if (areValidationErrors(req, res))
            return;
        return next();
    }
];
export const videoChannelsRemoveValidator = [
    async (req, res, next) => {
        if (!await checkVideoChannelIsNotTheLastOne(res.locals.videoChannel, res))
            return;
        return next();
    }
];
export const videoChannelsHandleValidatorFactory = (options) => {
    const { checkIsLocal, checkManage } = options;
    return [
        param('handle')
            .exists(),
        async (req, res, next) => {
            if (areValidationErrors(req, res))
                return;
            if (!await doesChannelHandleExist({ handle: req.params.handle, checkManage, checkIsLocal, res }))
                return;
            return next();
        }
    ];
};
export const ensureChannelOwnerCanUpload = [
    async (req, res, next) => {
        const channel = res.locals.videoChannel;
        const user = { id: channel.Account.userId };
        if (!await checkUserQuota(user, 1, res))
            return;
        next();
    }
];
export const videoChannelStatsValidator = [
    query('withStats')
        .optional()
        .customSanitizer(toBooleanOrNull)
        .custom(isBooleanValid).withMessage('Should have a valid stats flag boolean'),
    (req, res, next) => {
        if (areValidationErrors(req, res))
            return;
        return next();
    }
];
export const videoChannelsListValidator = [
    query('search')
        .optional()
        .not().isEmpty(),
    (req, res, next) => {
        if (areValidationErrors(req, res))
            return;
        return next();
    }
];
export const videoChannelImportVideosValidator = [
    body('externalChannelUrl')
        .custom(isUrlValid),
    body('videoChannelSyncId')
        .optional()
        .custom(isIdValid),
    async (req, res, next) => {
        if (areValidationErrors(req, res))
            return;
        const body = req.body;
        if (!CONFIG.IMPORT.VIDEOS.HTTP.ENABLED) {
            return res.fail({
                status: HttpStatusCode.FORBIDDEN_403,
                message: 'Channel import is impossible as video upload via HTTP is not enabled on the server'
            });
        }
        if (body.videoChannelSyncId && !await doesVideoChannelSyncIdExist(body.videoChannelSyncId, res))
            return;
        if (res.locals.videoChannelSync && res.locals.videoChannelSync.videoChannelId !== res.locals.videoChannel.id) {
            return res.fail({
                status: HttpStatusCode.FORBIDDEN_403,
                message: 'This channel sync is not owned by this channel'
            });
        }
        return next();
    }
];
async function checkVideoChannelIsNotTheLastOne(videoChannel, res) {
    const count = await VideoChannelModel.countByAccount(videoChannel.Account.id);
    if (count <= 1) {
        res.fail({
            status: HttpStatusCode.CONFLICT_409,
            message: 'Cannot remove the last channel of this user'
        });
        return false;
    }
    return true;
}
//# sourceMappingURL=video-channels.js.map